require("toCSV")

local debug=false
-----------------------------------
---------- Join and init ----------
-----------------------------------

script.on_event(defines.events.on_player_joined_game, function(event)
	local player_index = event.player_index
	joinHandler(player_index)
	printAll("Joined: " .. global.players[player_index].name .. " is player No. " .. global.players[player_index].index, true)
end)

script.on_event(defines.events.on_player_created, function(event)
	local player_index = event.player_index
	joinHandler(player_index)
	printAll("Created: " .. global.players[player_index].name .. " is player No. " .. global.players[player_index].index, true)
end)




function joinHandler(player_index)
	local player = game.players[player_index] or game.players[1]
	local name = player.name
	if not global then global = {} end
	if not global.players then global.players = {} end
	if not global.players[player_index] then 
		global.players[player_index] = initPlayer(player)
	end
	if (global.mustBelogged == nil) then 
		global.mustBelogged = false
	end
end


function initPlayer(player)
	if not player then 
		printAll("init Player begin Error!") 
		player = game.players[1]
	end
		
	local NewPlayer = {
		name = player.name,
		position = {x = player.position.x, y = player.position.y},
		index = player.index,
		buildedOwn = 0,
		destroyedOwn = 0,
		destroyedNotOwn = 0,
		markedDeconstruct = 0,
		canceledDeconst = 0,
		goodGuy = 0,
		badGuy = 0,
		tag = player.tag,
		mustBelogged = false,
		massiveDeconstructed = 0,
		online_time = 0,
		afk_time = 0,
		connected = true
		}
	-- there is no player in global!
	local text = player.name .. "[".. player.index .. "]"
	printAll("Init " .. text .. " - OK.")
	return NewPlayer
end

-----------------------------------
-------------- Stuff --------------
-----------------------------------

function printAll(text, always)
	if always or debug then
		for player_index in pairs(game.players) do
			game.players[player_index].print(text)
		end
	end
end


script.on_event(defines.events.on_tick, function(event)
	if (game.tick % (60*60*15) == 0) then --every 15 min
		printAll("saving timer")
		writeToCurrentCSV()
		for player_index in pairs (global.players) do
			writeOnExit(player_index) 
			global.players[player_index].massiveDeconstructed = 0
		end
	end
	if (global.mustBelogged)then
		for player_index in pairs (global.players) do
			if global.players[player_index].mustBelogged then
				printAll(playerInfo(player_index) .. " massiveDeconstructed:" .. global.players[player_index].massiveDeconstructed .. " Entities", true)
				writeStatistic(player_index)
				global.players[player_index].massiveDeconstructed = 0
				global.players[player_index].mustBelogged = false
			end
		end
	global.mustBelogged = false
	end
end)


-----------------------------------
------------ Good guy -------------
-----------------------------------

script.on_event(defines.events.on_built_entity, function(event)
	local player_index = event.player_index
	
	if not (global) then 
		printAll("There is no global")
		return
	end
	
	if not (global.players) then 
		printAll("There is no players")
		return
	end
	
	if not (global.players[player_index]) then 
		printAll("There is no player_index")
		return
	end
	
	global.players[player_index].buildedOwn = global.players[player_index].buildedOwn + 1
	printAll(playerInfo(player_index) .. " buildedOwn:" .. global.players[player_index].buildedOwn)

end)

script.on_event(defines.events.on_canceled_deconstruction, function(event)
	local player_index = event.player_index
	if not (global) then 
		printAll("There is no global")
		return
	end
	
	if not (global.players) then 
		printAll("There is no players")
		return
	end
	
	if not (global.players[player_index]) then 
		printAll("There is no player_index")
		return
	end
	
	global.players[player_index].canceledDeconst = global.players[player_index].canceledDeconst + 1
	printAll(playerInfo(player_index) .. " canceledDeconst:" .. global.players[player_index].canceledDeconst)
end)

-----------------------------------
------------ Bad guy --------------
-----------------------------------

script.on_event(defines.events.on_marked_for_deconstruction, function(event)
	local player_index = event.player_index
	local entity = event.entity
	if not entity.last_user then return end
	printAll("Last user: " .. entity.last_user.index)

	------------------------------------------
--	-- here: last_user.index is my index, not from that mat, who was last user. I hope it's coming in 0.15, but not now. 
	------------------------------------------
--	if isBad(player_index, entity) then	
		printAll("Bad")
		global.players[player_index].markedDeconstruct = global.players[player_index].markedDeconstruct + 1
		global.players[player_index].massiveDeconstructed = global.players[player_index].massiveDeconstructed + 1 
		printAll(playerInfo(player_index) .. " markedDeconstruct:" .. global.players[player_index].markedDeconstruct)
		-- next tick goes to log file, then we have only one string, not array
		global.mustBelogged = true
		global.players[player_index].mustBelogged = true
--	else
--		global.players[player_index].destroyedOwn = global.players[player_index].destroyedOwn + 1
--	end
end)


script.on_event(defines.events.on_preplayer_mined_item, function(event)
	local player_index = event.player_index
	local entity = event.entity
	if not entity.last_user then return end
	if entity.name == "entity-ghost" then return end
	local text = ""
	
	if isBad(player_index, entity) then 
		-- is bad boy
		global.players[player_index].destroyedNotOwn = global.players[player_index].destroyedNotOwn + 1
		text = playerInfo(player_index) .. "destroyedNotOwn:" .. global.players[player_index].destroyedNotOwn
		-- writeStatistic(player_index)
		global.players[player_index].mustBelogged = true
		printAll(text)
	else
		-- is good boy
		global.players[player_index].destroyedOwn = global.players[player_index].destroyedOwn + 1
		text =  playerInfo(player_index) .. 
			" destroyedOwn:" .. global.players[player_index].destroyedOwn
		printAll(text)
	end
end)

-----------------------------------
------------- Stuff 2 -------------
-----------------------------------

function isBad(player_index, entity)
	 if not (entity.last_user) or not (entity.last_user.index) then 
		printAll("no last_user")
		return false
	 end
	if not (player_index == entity.last_user.index) then
		local text = playerInfo(player_index) .." has removed '" .. entity.name .. "' of " .. entity.last_user.name .. "[" .. entity.last_user.index .."]"
		printAll(text, true)
		return true
	else
		return false	
	end
end

function playerInfo(player_index)
	return global.players[player_index].name .. "[".. global.players[player_index].index .. "]"
end

-----------------------------------
------------ Logging --------------
-----------------------------------

script.on_event(defines.events.on_player_left_game, function(event)
	local player_index = event.player_index
	writeOnExit(player_index)
	writeStatistic(player_index)
end)


function writeStatistic(player_index)

	updateStatistic (player_index)
	local player = global.players[player_index]
	-- darkfrei: there is now way to get time or some game ID, then it will be this tick. Tell me if it's possible.
	local dateGame = game.tick
	local data = dateGame .. " "
	local badGuy = player.badGuy
	local goodGuy = player.goodGuy
	if not (player.massiveDeconstructed == 0) then
		data = dateGame .. "massiveDeconstructed:" .. player.massiveDeconstructed .. " \n"
	end
	
	
	local filename = "build-statistic/statistic_" .. playerInfo(player_index) .. ".log"
	data = data .. dateGame .. " " .. playerInfo(player_index) .. " is a bad guy:" .. badGuy .. ", a good guy:" .. goodGuy
	
	for index in pairs (game.players) do
		game.write_file(filename, data .. " \n", true, index)
	end
	
	writeToCurrentCSV()
end

function updateStatistic (player_index)
	local p = game.players[player_index]
	local player = global.players[player_index]

	global.players[player_index].badGuy = player.destroyedNotOwn + player.markedDeconstruct 
	global.players[player_index].goodGuy = player.buildedOwn + player.canceledDeconst + player.destroyedOwn
	
	global.players[player_index].position = {x = player.position.x, y = player.position.y}
	
	global.players[player_index].online_time = p.online_time
	global.players[player_index].afk_time = p.afk_time
	global.players[player_index].connected = p.connected

end

	
function writeToCurrentCSV()
	if not global then printAll("no global!") end
	
	local s = toCSV (global.players)
	if s then 
		s =  s .. " \n end" 
	else 
		s = " \n bad end"
	end
	
	-- save to all players
	local filename = "build-statistic/current_log.csv"
	
	for index in pairs (game.players) do
		game.write_file(filename, s, false, index)
	end
	
	--save to server
	game.write_file(filename, s, false, 0)
end
	

	
	
function writeOnExit(player_index)
	-- darkfrei: there is now way to get time or some game ID, then it will be this tick. Tell me if it's possible.
	local dateGame = game.tick
	local data = dateGame .. " Statistic: \n"
	data = data .. dateGame .. " Bad Guy while:" .. 
		" destroyedNotOwn:" .. global.players[player_index].destroyedNotOwn ..
		" markedDeconstruct:" .. global.players[player_index].markedDeconstruct .. "  \n"
	data = data .. dateGame .. " Good Guy while:" .. 
		" buildedOwn:" .. global.players[player_index].buildedOwn ..
		" canceledDeconst:" .. global.players[player_index].canceledDeconst .. 
		" destroyedNotOwn:" .. global.players[player_index].destroyedNotOwn .. " \n"
	
	local massiveDeconstructed = global.players[player_index].massiveDeconstructed
	if not (massiveDeconstructed == 0) then
		data = data .. dateGame .. " And massive deconstructed " .. global.players[player_index].destroyedNotOwn .. " Entities \n"
	end
	local filename = "build-statistic/statistic_" .. playerInfo(player_index) .. ".log"
	-- save for all, not server only
	for index in pairs (game.players) do
		game.write_file(filename, data .. " \n", true, index)
	end
	printAll(filename)
end



	-- on_pre_player_died
	-- Called before a players dies.

	-- Contains
	-- player_index :: uint
	
	
	
	-- on_entity_died
	-- Called when an entity dies.

	-- Contains
	-- entity :: LuaEntity
	-- force :: LuaForce (optional): The force that did the killing if any.
	